import unittest

from bankline_parser.data_services import enums, fields
from bankline_parser.data_services.exceptions import ParseError


class DataServicesFieldsTestCase(unittest.TestCase):
    def test_text_field(self):
        row = 'amazing'
        text_field = fields.TextField(0, 7)
        self.assertEqual(text_field.parse(row), 'amazing')

    def test_text_field_with_padding(self):
        row = '   amazing'
        text_field = fields.TextField(0, 10)
        self.assertEqual(text_field.parse(row), 'amazing')

    def test_text_field_with_padding_left_justified(self):
        row = 'amazing   '
        text_field = fields.TextField(0, 10, justification='l')
        self.assertEqual(text_field.parse(row), 'amazing')

    def test_text_field_left_justified_left_padding_not_removed(self):
        row = '   amazing'
        text_field = fields.TextField(0, 10, justification='l')
        self.assertEqual(text_field.parse(row), '   amazing')

    def test_numeric_field(self):
        row = 'otherstuff000064'
        numeric_field = fields.NumericField(10, 16)
        self.assertEqual(numeric_field.parse(row), 64)

    def test_numeric_field_nonnumeric_value_fails(self):
        row = 'h67'
        numeric_field = fields.NumericField(0, 3)
        try:
            numeric_field.parse(row)
            self.fail()
        except ParseError:
            pass

    def test_enum_field(self):
        row = 'gggggD000000'
        enum_field = fields.EnumField(5, 6, enums.BalanceType)
        self.assertEqual(enum_field.parse(row), enums.BalanceType.debit)

    def test_static_field(self):
        row = 'hello'
        static_field = fields.StaticField(0, 6, 'hello')
        self.assertEqual(static_field.parse(row), 'hello')

    def test_static_field_trim_padding(self):
        row = '   hello'
        static_field = fields.StaticField(0, 9, 'hello')
        self.assertEqual(static_field.parse(row), 'hello')

    def test_static_field_include_padding(self):
        row = '   hello'
        static_field = fields.StaticField(0, 9, '   hello', pad_char=None)
        self.assertEqual(static_field.parse(row), '   hello')

    def test_static_field_incorrect_value_fails(self):
        row = 'herro'
        static_field = fields.StaticField(0, 6, 'hello')
        try:
            static_field.parse(row)
            self.fail()
        except ParseError:
            pass



from datetime import datetime

from .exceptions import ParseError


class DataField:
    def __init__(self, start, end, justification='r', fill_char=' ', pad_char=' '):
        self.start = start
        self.end = end
        self.justification = justification
        self.fill_char = fill_char
        self.pad_char = pad_char

    def _strip_padding(self, field_content):
        if self.justification == 'r':
            for i in range(len(field_content)):
                if field_content[i] == self.pad_char:
                    continue
                else:
                    return field_content[i:]
        else:
            for i in reversed(range(len(field_content))):
                if field_content[i] == self.pad_char:
                    continue
                else:
                    return field_content[:i+1]

    def parse(self, row_content):
        field_content = row_content[self.start:self.end]
        if field_content == self.fill_char*(self.end-self.start):
            return None
        return self._strip_padding(field_content)


class TextField(DataField):
    pass


class DateField(DataField):
    def parse(self, row_content):
        field_content = row_content[self.start:self.end]
        try:
            return datetime.strptime(field_content, ' %y%j')
        except ValueError as e:
            raise ParseError(e)


class NumericField(DataField):
    def parse(self, row_content):
        field_content = super().parse(row_content)
        if field_content:
            try:
                return int(field_content)
            except (TypeError, ValueError) as e:
                raise ParseError(e)
        else:
            return None


class ZeroFilledField(DataField):
    def __init__(self, start, end, fill_char='0', **kwargs):
        super().__init__(start, end, fill_char=fill_char, **kwargs)


class EnumField(DataField):
    def __init__(self, start, end, enum, **kwargs):
        self.enum = enum
        super().__init__(start, end, **kwargs)

    def parse(self, row_content):
        field_content = row_content[self.start:self.end]
        try:
            return self.enum(field_content)
        except ValueError as e:
            raise ParseError(e)


class StaticField(DataField):
    def __init__(self, start, end, value, **kwargs):
        self.value = value
        super().__init__(start, end, **kwargs)

    def parse(self, row_content):
        field_content = super().parse(row_content)
        if field_content == self.value:
            return field_content
        else:
            raise ParseError(
                "Field content '%s' does not match expected static value '%s'"
                % (field_content, self.value)
            )



class IndexTrackingIterator:
    def __init__(self, lines):
        self.iterator = iter(lines)
        self.current_index = 0

    def __next__(self):
        item = next(self.iterator)
        self.current_index += 1
        return item



import unittest

from bankline_parser.data_services import parse
from bankline_parser.data_services.exceptions import ParseError

from .test_models import vhl_row


class DataServicesParserTestCase(unittest.TestCase):
    def test_nwb_testfile_successful_parse(self):
        with open('tests/data/testfile_nwb') as f:
            output = parse(f)

        self.assertTrue(output.is_valid())
        self.assertEqual(len(output.accounts), 1)
        self.assertEqual(len(output.account.records), 29)
        self.assertEqual(
            output.account.user_trailer_label.monetary_total_debit_items,
            278225388
        )
        self.assertEqual(
            output.account.user_trailer_label.monetary_total_credit_items,
            105598692
        )

    def test_rbs_testfile_successful_parse(self):
        with open('tests/data/testfile_rbs') as f:
            output = parse(f)

        self.assertTrue(output.is_valid())
        self.assertEqual(len(output.accounts), 1)
        self.assertEqual(len(output.account.records), 25)
        self.assertEqual(
            output.account.user_trailer_label.monetary_total_debit_items,
            554186444
        )
        self.assertEqual(
            output.account.user_trailer_label.monetary_total_credit_items,
            666903844
        )

    def test_empty_file_fails(self):
        try:
            parse([])
            self.fail()
        except ParseError:
            pass

    def test_no_accounts_fails(self):
        try:
            parse([vhl_row])
            self.fail()
        except ParseError:
            pass



#!/usr/bin/env python
import sys
import warnings

from setuptools import setup

if sys.version_info[0:2] < (3, 7):
    warnings.warn('This package is only tested on Python version 3.7+', stacklevel=1)

setup()



import unittest
from datetime import datetime

from bankline_parser.data_services import enums, models

vhl_row = (
    'VOL1                                 ****830000                '
    '                3                                                '
)
fhl_row = (
    'HDR1 A876543Z0001                      F     01 06032          '
    '    060320                                                       '
)
uhl_row = (
    'UHL1 06032876543    00000000         000              Datalink '
    'Test File                                                          '
)
data_record = (
    '8765438765432101112345678418953000000000006086                 '
    ' 123456                               06032                      '
)
balance_record = (
    '123456671753150Y1              0000 000000038510000C0000000000000'
    '00C000000056571776C000000038474276C 04036                      '
)
utl_row = (
    'UTL10000554186444000066690384400000150000010                   '
    '                                                                 '
)


class DataServicesRowsTestCase(unittest.TestCase):
    def test_volume_header_label(self):
        vhl = models.VolumeHeaderLabel(vhl_row)
        self.assertEqual(vhl.label_identifier, 'VOL1')
        self.assertEqual(vhl.volume_serial_number, None)
        self.assertEqual(vhl.owner_identification, '****830000')
        self.assertEqual(vhl.label_standard_level, 3)

    def test_file_header_label(self):
        fhl = models.FileHeaderLabel(fhl_row)
        self.assertEqual(fhl.label_identifier, 'HDR1')
        self.assertEqual(fhl.file_identifier, 'A876543Z')
        self.assertEqual(fhl.file_sequence_number, 1)
        self.assertEqual(fhl.record_format, 'F')
        self.assertEqual(fhl.creation_date, datetime.strptime('06032', '%y%j'))
        self.assertEqual(fhl.expiration_date, datetime.strptime('06032', '%y%j'))
        self.assertEqual(fhl.copy_indicator, None)

    def test_user_header_level(self):
        uhl = models.UserHeaderLabel(uhl_row)
        self.assertEqual(uhl.label_identifier, 'UHL1')
        self.assertEqual(uhl.processing_date, datetime.strptime('06032', '%y%j'))
        self.assertEqual(uhl.branch_sort_code, '876543')
        self.assertEqual(uhl.currency_code, None)
        self.assertEqual(uhl.work_code, None)
        self.assertEqual(uhl.file_number, None)
        self.assertEqual(uhl.customer_name, 'Datalink Test File')

    def test_data_record(self):
        dr = models.DataRecord(data_record)
        self.assertEqual(dr.branch_sort_code, '876543')
        self.assertEqual(dr.branch_account_number, '87654321')
        self.assertEqual(dr.type_of_account_code, None)
        self.assertEqual(dr.transaction_code, enums.TransactionCode.debit_cheque)
        self.assertEqual(dr.originators_sort_code, '123456')
        self.assertEqual(dr.originators_account_number, '78418953')
        self.assertEqual(dr.originators_reference, None)
        self.assertEqual(dr.amount, 6086)
        self.assertEqual(dr.transaction_description, None)
        self.assertEqual(dr.reference_number, '123456            ')
        self.assertEqual(dr.date, datetime.strptime('06032', '%y%j'))

    def test_balance_record(self):
        br = models.BalanceRecord(balance_record)
        self.assertEqual(br.branch_sort_code, '123456')
        self.assertEqual(br.branch_account_number, '67175315')
        self.assertEqual(br.type_of_account_code, None)
        self.assertEqual(br.transaction_code, enums.TransactionCode.balance_record)
        self.assertEqual(br.ledger_balance, 38510000)
        self.assertEqual(br.ledger_balance_type, enums.BalanceType.credit)
        self.assertEqual(br.cleared_balance, 0)
        self.assertEqual(br.cleared_balance_type, enums.BalanceType.credit)
        self.assertEqual(br.recorded_ledger_balance, 56571776)
        self.assertEqual(br.recorded_ledger_balance_type, enums.BalanceType.credit)
        self.assertEqual(br.recorded_cleared_balance, 38474276)
        self.assertEqual(br.recorded_cleared_balance_type, enums.BalanceType.credit)
        self.assertEqual(br.date, datetime.strptime('04036', '%y%j'))

    def test_user_trailer_label(self):
        utl = models.UserTrailerLabel(utl_row)
        self.assertEqual(utl.label_identifier, 'UTL1')
        self.assertEqual(utl.monetary_total_debit_items, 554186444)
        self.assertEqual(utl.monetary_total_credit_items, 666903844)
        self.assertEqual(utl.count_debit_items, 15)
        self.assertEqual(utl.count_credit_items, 10)
        self.assertEqual(utl.count_balance_records, None)
